# -*- coding: utf-8 -*-

import logging

from .common import Rect


class Layout:
    def _getchildren(self):
        return [child for child in self.container.children
            if child.allowlayout and not child.hidden]

    def _getregion(self):
        c = self.container
        bl, bt, br, bb = c.borders
        return Rect(bl, bt, c.width - bl - br, c.height - bt - bb)


class VerticalLayout(Layout):
    def resize(self):
        log = logging.getLogger('tuikit')
        log.debug('VerticalLayout: resize %d children', len(self._getchildren()))

        v = 0
        c = self.container
        bl, bt, br, bb = c.borders

        last = None
        for child in self._getchildren():
            child.x = bl
            child.width = c.width - bl - br
            child.y = bt + v
            v += child.height
            child.handle('resize')
            last = child
            
            log.debug('VerticalLayout: child %r to y=%d', child, child.y)

        if last and v < c.height - bt - bb:
            last.height += c.height - bt - bb - v

        c.redraw()


class HorizontalLayout(Layout):
    def resize(self):
        log = logging.getLogger('tuikit')
        log.debug('HorizontalLayout: resize')
        
        v = 0
        c = self.container
        bl, bt, br, bb = c.borders

        last = None
        for child in self._getchildren():
            child.y = bt
            child.height = c.height - bt - bb
            child.x = bl + v
            v += child.width
            child.handle('resize')
            last = child
        
        if last and v < c.width - bl - br:
            last.width += c.width - bl - br - v
        
        c.redraw()


class GridLayout(Layout):
    def __init__(self, numcols=2):
        self.numcols = numcols


    def _fillgrid(self):
        ''' fill grid with widgeds '''
        self._grid = []
        rown = 0
        coln = 0
        for child in self._getchildren():
            if coln == 0:
                row = []
                for i in range(self.numcols):
                    row.append({'widget': None, 'colspan': 0, 'rowspan': 0})

            colspan = 1
            if 'colspan' in child.hints:
                colspan = child.hints['colspan']
            colspan = min(colspan, self.numcols - coln + 1)

            row[coln]['widget'] = child
            row[coln]['colspan'] = colspan

            coln += colspan
            if coln >= self.numcols:
                coln = 0
                self._grid.append(row)
                rown += 1

        # autospan last child
        if coln > 0:
            row[coln-1]['colspan'] = self.numcols - coln + 1
            self._grid.append(row)


    def _computesizes(self):
        self._colminw = [0] * self.numcols

        # compute min column width for all widgets with colspan = 1
        for row in self._grid:
            for coln in range(self.numcols):
                w = row[coln]['widget']
                if row[coln]['colspan'] == 1:
                    self._colminw[coln] = max(w.sizemin[0], self._colminw[coln])

        # adjust min width for widgets with colspan > 1
        for row in self._grid:
            for coln in range(self.numcols):
                w = row[coln]['widget']
                colspan = row[coln]['colspan']
                if colspan > 1:
                    # find min width over spanned columns
                    totalminw = 0
                    for i in range(colspan):
                        totalminw += self._colminw[coln + i]
                    # check if spanned widget fits in
                    if w.sizemin[0] > totalminw:
                        # need to adjust colminw
                        addspace = w.sizemin[0] - totalminw
                        addall = addspace // colspan
                        rest = addspace % colspan
                        for i in range(colspan):
                            self._colminw[coln + i] += addall
                            if i < rest:
                                self._colminw[coln + i] += 1

        self._rowminh = [0] * len(self._grid)
        rown = 0
        for row in self._grid:
            for col in row:
                w = col['widget']
                if w is not None:
                    self._rowminh[rown] = max(self._rowminh[rown], w.sizemin[1])
            rown += 1

        self._gridminw = sum(self._colminw)
        self._gridminh = sum(self._rowminh)


    def resize(self):
        self._fillgrid()
        self._computesizes()

        # enlarge container if needed
        lreg = self._getregion() # layout region
        cont = self.container
        bl, bt, br, bb = cont.borders
        if self._gridminw > lreg.w:
            cont.width = self._gridminw + bl + br
        if self._gridminh > lreg.h:
            cont.height = self._gridminh + bt + bb

        # compute actual width of columns
        colw = [0] * self.numcols
        lreg = self._getregion() # layout region
        restw = lreg.w - self._gridminw
        resth = lreg.h - self._gridminh
        for c in range(self.numcols):
            colw[c] = self._colminw[c] + restw // self.numcols
            if c < restw % self.numcols:
                colw[c] += 1

        # place widgets
        colx = lreg.x
        coly = lreg.y
        rown = 0
        numrows = len(self._grid)
        for row in self._grid:
            coln = 0
            rowh = self._rowminh[rown] + resth // numrows
            if rown < resth % numrows:
                rowh += 1

            for col in row:
                w = col['widget']
                if w is not None:
                    w.x = colx
                    w.y = coly
                    w.width = colw[coln]
                    w.height = rowh
                    w.handle('resize')

                    colspan = col['colspan']
                    if colspan > 1:
                        for i in range(1,colspan):
                            w.width += colw[coln + i]

                colx += colw[coln]

            rown += 1
            colx = lreg.x
            coly += rowh

        cont.redraw()

